@@ -29,8 +29,6 @@ from netsblox import common
 
 NETSBLOX_PY_PATH = os.path.dirname(netsblox.__file__)
 
-IMG_ROOT = 'https://raw.githubusercontent.com/dragazo/NetsBlox-python/master/img'
-
 SUGGESTION_UPDATE_INTERVAL = 200
 
 xux = lambda x: f'{x} {x.upper()}'
@@ -230,28 +228,6 @@ def clean_docstring(content: str) -> str:
     res = re.sub(INLINE_CODE_REGEX, r'\1', res)
     return res
 
-_img_cache = {}
-_error_image = Image.new('RGB', (50, 50), (252, 3, 244))
-def load_image(img_url: str, *, scale: float = 1):
-    if img_url in _img_cache:
-        return _img_cache[img_url]
-
-    res = requests.get(img_url)
-    img = _error_image # defualt to error image
-    if res.status_code == 200:
-        img = Image.open(io.BytesIO(res.content))
-    else:
-        title = 'Failed to Load Image'
-        msg = f'Failed to load image {img_url} (error code {res.status_code})\n\nMake sure the web host allows direct downloads.'
-        messagebox.showerror(title, msg)
-
-    if scale != 1:
-        img = img.resize((round(img.width * scale), round(img.height * scale)))
-
-    img = ImageTk.PhotoImage(img)
-    _img_cache[img_url] = img
-    return img
-
 _exec_monitor_running = False
 def start_exec_monitor():
     global _exec_monitor_running, _exec_process
@@ -410,7 +386,7 @@ class BlocksList(tk.Frame):
 
         self.imgs = [] # for some reason we need to keep a reference to the images or they disappear
         for block in blocks:
-            img = load_image(block['url'], scale = block['scale'])
+            img = common.load_tkimage(block['url'], scale = block['scale'])
             label = tk.Label(self.text, image = img, bg = COLOR_INFO['text-background-disabled'])
 
             self.text.window_create('end', window = label)
@@ -678,92 +654,16 @@ class ProjectEditor(tk.Frame):
         scripts.append('start_project()')
         return ''.join(scripts)
 
-    DEFAULT_GLOBAL_BLOCKS = [
-        { 'url': f'{IMG_ROOT}/blocks/onstart.png', 'scale': 1, 'replace': '@onstart()\ndef my_onstart(): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/keypress.png', 'scale': 1, 'replace': '@onkey(\'space\')\ndef my_onkey(): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/msgrecv-snap.png', 'scale': 1, 'replace': '@nb.on_message(\'message\')\ndef my_on_message(msg): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/sendmsg-snap.png', 'scale': 1, 'replace': 'nb.send_message(\'message\', \'myself\', msg = \'Hello World!\')' },
-        { 'url': f'{IMG_ROOT}/blocks/loop-forever.png', 'scale': 1, 'replace': 'while True:\n    pass # replace with your code' },
-
-        { 'url': f'{IMG_ROOT}/blocks/warp.png', 'scale': 1, 'replace': 'with Warp():\n    pass # replace with your code' },
-
-        { 'url': f'{IMG_ROOT}/blocks/msgrecv.png', 'scale': 1, 'replace': '@nb.on_message(\'message\')\ndef my_on_message(msg): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/sendmsg.png', 'scale': 1, 'replace': 'nb.send_message(\'message\', \'myself\', msg = \'Hello World!\')' },
-        { 'url': f'{IMG_ROOT}/blocks/loop-foreach.png', 'scale': 1, 'replace': 'for item in my_list:\n    pass # replace with your code' },
-    ]
-    DEFAULT_STAGE_BLOCKS = [
-        { 'url': f'{IMG_ROOT}/blocks/onstart.png', 'scale': 1, 'replace': '@onstart()\ndef my_onstart(self): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/keypress.png', 'scale': 1, 'replace': '@onkey(\'space\')\ndef my_onkey(self): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/onclick.png', 'scale': 1, 'replace': '@onclick()\ndef my_onclick(self, x, y): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/msgrecv-snap.png', 'scale': 1, 'replace': '@nb.on_message(\'message\')\ndef my_on_message(self, msg): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/sendmsg-snap.png', 'scale': 1, 'replace': 'nb.send_message(\'message\', \'myself\', msg = \'Hello World!\')' },
-        { 'url': f'{IMG_ROOT}/blocks/loop-forever.png', 'scale': 1, 'replace': 'while True:\n    pass # replace with your code' },
-
-        { 'url': f'{IMG_ROOT}/blocks/warp.png', 'scale': 1, 'replace': 'with Warp():\n    pass # replace with your code' },
-
-        { 'url': f'{IMG_ROOT}/blocks/msgrecv.png', 'scale': 1, 'replace': '@nb.on_message(\'message\')\ndef my_on_message(self, msg): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/sendmsg.png', 'scale': 1, 'replace': 'nb.send_message(\'message\', \'myself\', msg = \'Hello World!\')' },
-        { 'url': f'{IMG_ROOT}/blocks/loop-foreach.png', 'scale': 1, 'replace': 'for item in my_list:\n    pass # replace with your code' },
-    ]
-    DEFAULT_TURTLE_BLOCKS = [
-        { 'url': f'{IMG_ROOT}/blocks/onstart.png', 'scale': 1, 'replace': '@onstart()\ndef my_onstart(self): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/onstartclone.png', 'scale': 1, 'replace': '@onstart(when = \'clone\')\ndef my_onstartclone(self): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/keypress.png', 'scale': 1, 'replace': '@onkey(\'space\')\ndef my_onkey(self): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/onclick.png', 'scale': 1, 'replace': '@onclick()\ndef my_onclick(self, x, y): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/msgrecv-snap.png', 'scale': 1, 'replace': '@nb.on_message(\'message\')\ndef my_on_message(self, msg): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/sendmsg-snap.png', 'scale': 1, 'replace': 'nb.send_message(\'message\', \'myself\', msg = \'Hello World!\')' },
-        { 'url': f'{IMG_ROOT}/blocks/loop-forever.png', 'scale': 1, 'replace': 'while True:\n    pass # replace with your code' },
-
-        { 'url': f'{IMG_ROOT}/blocks/warp.png', 'scale': 1, 'replace': 'with Warp():\n    pass # replace with your code' },
-
-        { 'url': f'{IMG_ROOT}/blocks/msgrecv.png', 'scale': 1, 'replace': '@nb.on_message(\'message\')\ndef my_on_message(self, msg): # functions need different names\n    pass # replace with your code' },
-        { 'url': f'{IMG_ROOT}/blocks/sendmsg.png', 'scale': 1, 'replace': 'nb.send_message(\'message\', \'myself\', msg = \'Hello World!\')' },
-        { 'url': f'{IMG_ROOT}/blocks/loop-foreach.png', 'scale': 1, 'replace': 'for item in my_list:\n    pass # replace with your code' },
-    ]
-    DEFAULT_PROJECT = {
-        'global_blocks': DEFAULT_GLOBAL_BLOCKS,
-        'stage_blocks': DEFAULT_STAGE_BLOCKS,
-        'turtle_blocks': DEFAULT_TURTLE_BLOCKS,
-        'show_blocks': True,
-        'editors': [
-            {
-                'type': 'global',
-                'name': 'global',
-                'value': '''
-my_global_var = 'hello world' # create a global variable
-'''.lstrip(),
-            },
-            {
-                'type': 'stage',
-                'name': 'stage',
-                'value': '''
-@onstart
-def my_onstart(self): # functions need different names
-    self.my_list = [1, 2, 3] # create a stage variable
-'''.lstrip(),
-            },
-            {
-                'type': 'turtle',
-                'name': 'sprite',
-                'value': '''
-@onstart
-def my_onstart(self): # functions need different names
-    self.my_distance = 2 # create a sprite variable
-
-    for i in range(360): # repeat code 360 times
-        self.forward(self.my_distance)
-        self.turn_right(1)
-'''.lstrip(),
-            },
-        ],
-        'imports': [],
-    }
+    DEFAULT_PROJECT = json.loads(common.load_text('netsblox://assets/default-project.json'))
 
     def get_save_dict(self) -> dict:
         res = {}
-        res['global_blocks'] = [x.copy() for x in GlobalEditor.blocks]
-        res['stage_blocks'] = [x.copy() for x in  StageEditor.blocks]
-        res['turtle_blocks'] = [x.copy() for x in TurtleEditor.blocks]
+        res['blocks'] = {
+            'sources': self.block_sources[:],
+            'global': [x.copy() for x in GlobalEditor.blocks if x['source'] == None],
+            'stage':  [x.copy() for x in  StageEditor.blocks if x['source'] == None],
+            'turtle': [x.copy() for x in TurtleEditor.blocks if x['source'] == None],
+        }
         res['show_blocks'] = self.show_blocks
         res['imports'] = []
         for pkg, item in self.imports.packages.items():
@@ -784,9 +684,30 @@ def my_onstart(self): # functions need different names
         res['images'] = { name: common.encode_image(img) for name, img in self.imports.images.items() }
         return res
     def load(self, proj: dict) -> None:
-        GlobalEditor.blocks = [x.copy() for x in proj['global_blocks']]
-        StageEditor.blocks = [x.copy() for x in proj['stage_blocks']]
-        TurtleEditor.blocks = [x.copy() for x in proj['turtle_blocks']]
+        new_blocks = { 'global': [], 'stage': [], 'turtle': [] }
+        new_sources = proj.get('blocks', {}).get('sources', [])
+
+        def add_blocks(blocks, *, source):
+            for k, v in blocks.items():
+                target = new_blocks[k]
+                for block in v:
+                    x = block.copy()
+                    x['source'] = source
+                    target.append(x)
+
+        for src in new_sources:
+            add_blocks(json.loads(common.load_text(src)), source = src)
+        add_blocks({ k: v for k, v in proj.get('blocks', {}).items() if k in new_blocks }, source = None)
+        add_blocks({ # legacy support
+            'global': proj.get('global_blocks', []),
+            'stage': proj.get('stage_blocks', []),
+            'turtle': proj.get('turtle_blocks', []),
+        }, source = None)
+
+        GlobalEditor.blocks = new_blocks['global']
+        StageEditor.blocks = new_blocks['stage']
+        TurtleEditor.blocks = new_blocks['turtle']
+        self.block_sources = new_sources
 
         for i in range(len(self.editors) - 1, -1, -1):
             self.notebook.forget(i)
@@ -1659,7 +1580,7 @@ def main():
     style = ttk.Style(root)
     style.configure('TNotebook', tabposition = 'n')
 
-    logo = load_image(f'{IMG_ROOT}/logo/logo-256.png')
+    logo = common.load_tkimage(f'netsblox://assets/img/logo/logo-256.png')
     root.iconphoto(True, logo)
 
     content = Content(root)
